



if not exist %PREFIX%\\Library\\include\\primesieve.h exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\\Library\\bin\\primesieve.dll exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
primesieve -n 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
